Attribute VB_Name = "bOpenSave"
'
' Name:         bOpenSave (bOpenSave.bas)
' Author:       Eric B. Stauffer
' Date:         04 May 1999
'
' Description:  The bOpenSave modules provides a wrapper to the Windows
'               API call GetOpenFileNameA and GetSaveFileNameA which are in COMDLG32.DLL.
'               Using this modules eliminates the need to include CMNDLG32.OCX in your
'               applications distributions _and_ frees you from having to place the control
'               on each form from which the common dialog is to be displayed. The only shortcoming
'               is that it is not possible to display the dialog box from Sub Main() if you
'               start your application in that fashion.
'
' Use:          Include the file bOpenSave in your project. The call requires only the two parms
'                   DIALOG_TYPE:    Indicates File/Open or File/Save
'                   hWnd:           Provide a display handle
'               A number of configurable parms as well as run-time parms can be defined. Those that
'               can be set as general program-level parms are defined as CONSTs and should be
'               quite self-explanatory. Call level parms are defined below.
'
' Call Parms:   Only one call level parms will be discussed at length here. All others mimic their
'               namesakes in the Common Dialog VB documentation. Of notes is that the FLAGS property
'               has been enumerated within the function to make calling simpler. You can OR these
'               together as in the common dialog control.
'
'               Specifying a filter is one of the more bothersome (if not cumbersome) tasks in VB.
'               A function within the bOpenSave (AddFilterConstraint) makes this less painful. Recall
'               that the 'Filter' parms tells the common dialog what files to display by default. Two
'               components comprise each filter: what the user sees in the program, and what is passed
'               to the common dialog. For example, when opening a project in VB, for example, the filter
'               is set to "Project Files *.vbp". Internally, the dialog is passed the string:
'
'                                   "Project Files *.vbp" + CHR(0) + "*.vbp" + CHR(0)
'
'               The AddFilterConstraint assists the programmber by building this string. You can call
'               the function repeatedly to 'build' a filter. On calls subsequent to the first, simply
'               pass in the existing filter as the last parm. This last example builds a filter for
'               perl programs and image files:
'
'               Dim sFilter as String
'               sFilter = AddFilterConstraint("Perl Programs (*.pl)","*.pl")
'               sFilter = AddFilterConstraint("Images","*.bmp;*.gif;*.jpg",sFilter)
'
'               Notice how the extension constraints are semi-colon delimted.
'






Option Explicit

Declare Function GetOpenFileName Lib "comdlg32.dll" Alias "GetOpenFileNameA" (OFN As OPENFILENAME) As Boolean
Declare Function GetSaveFileName Lib "comdlg32.dll" Alias "GetSaveFileNameA" (OFN As OPENFILENAME) As Boolean
Declare Function CommDlgExtendedError Lib "comdlg32.dll" () As Long

'
' _THE_ important structure
'

Type OPENFILENAME
    lStructSize As Long
    hwndOwner As Long
    hInstance As Long
    sFilter As String
    sCustomFilter As String
    nMaxCustFilter As Long
    nFilterIndex As Long
    sFile As String
    nMaxFile As Long
    sFileTitle As String
    nMaxFileTitle As Long
    sInitialDir As String
    sTitle As String
    Flags As Long
    nFileOffset As Integer
    nFileExtension As Integer
    sDefExt As String
    lCustData As Long
    lpfnHook As Long
    lpTemplateName As String
End Type

'
' Flags enumeration to make life simple on programmers
'

Enum OpenSaveFlags
    READ_ONLY = &H1
    OVERWRITE_PROMPT = &H2
    HIDE_READ_ONLY = &H4
    NO_CHANGE_DIR = &H8
    SHOW_HELP = &H10
    NO_VALIDATE = &H100
    ALLOW_MULTISELECT = &H200
    EXTENSION_DIFFERENT = &H400
    PATH_MUST_EXIST = &H800
    FILE_MUST_EXIST = &H1000
    CREATE_PROMPT = &H2000
    SHARE_AWARE = &H4000
    NO_READ_ONLY_RETURN = &H8000
    NO_TEST_FILE_CREATE = &H10000
    NO_NETWORK_BUTTON = &H20000
    NO_LONG_NAMES = &H40000
    EXPLORER = &H80000
    NO_DEREFERENCE_LINKS = &H100000
    LONGNAMES = &H200000
End Enum

Enum DIALOG_STYLE
    FILE_OPEN = True
    FILE_SAVE = False
End Enum

Public Const DEFAULT_FITLER As String = "All Files (*.*)" & vbNullChar & "*.*" & vbNullChar
Public Const DEFAULT_TITLE As String = "Specify File to Open"
Public Const DEFAULT_INIT_DIR As String = "C:\"
Public Const DEFAULT_FILE_OPEN_TITLE As String = "Specify file to open"
Public Const DEFAULT_FILE_SAVE_TITLE As String = "Specify file to save"

Function FileOpenSave(Dialog_Type As DIALOG_STYLE, _
                        hwnd As Long, _
                        Optional ByRef Flags As OpenSaveFlags, _
                        Optional ByVal InitialDir As Variant, _
                        Optional ByVal Filter As Variant, _
                        Optional ByVal FilterIndex As Variant, _
                        Optional ByVal DefaultExt As Variant, _
                        Optional ByVal FileName As Variant, _
                        Optional ByVal DialogTitle As Variant) As Variant
                        

    Dim OFN As OPENFILENAME

    Dim sFilename As String
    Dim sFileTitle As String
    Dim fResult As Boolean
        
    If IsMissing(Filter) Then Filter = DEFAULT_FITLER
    If IsMissing(FilterIndex) Then FilterIndex = 1
    If IsMissing(Flags) Then Flags = 0&
    If IsMissing(DefaultExt) Then DefaultExt = ""
    If IsMissing(FileName) Then FileName = ""
    
    If IsMissing(DialogTitle) Then
        If Dialog_Type Then
            DialogTitle = DEFAULT_FILE_OPEN_TITLE
        Else
            DialogTitle = DEFAULT_FILE_SAVE_TITLE
        End If
    End If
        
    '
    ' Allocate string space for the returned strings.
    '
    
    sFilename = Left(FileName & String(256, 0), 256)
    sFileTitle = String(256, 0)
    
    ' Set up the data structure before you call the function
    
    With OFN
        .lStructSize = Len(OFN)
        .hwndOwner = hwnd
        .sFilter = Filter
        .nFilterIndex = FilterIndex
        .sFile = sFilename
        .nMaxFile = Len(sFilename)
        .sFileTitle = sFileTitle
        .nMaxFileTitle = Len(sFileTitle)
        .sTitle = DialogTitle
        .Flags = Flags
        .sDefExt = DefaultExt
        .sInitialDir = DEFAULT_INIT_DIR
        .hInstance = 0
        .sCustomFilter = ""
        .nMaxCustFilter = 0
        .lpfnHook = 0
        .sCustomFilter = String(255, 0)
        .nMaxCustFilter = 255
    End With
    
    '
    ' Call the API function
    '
    
    If Dialog_Type Then
        fResult = GetOpenFileName(OFN)
    Else
        fResult = GetSaveFileName(OFN)
    End If

        
    If fResult Then
        If Not IsMissing(Flags) Then Flags = OFN.Flags
        FileOpenSave = SnipAtNull(OFN.sFile)
    Else
        FileOpenSave = vbNullString
    End If
    
End Function


Function AddFilterConstraint(sDescription As String, sMask As String, Optional sOldFilter As String) As String

    '
    ' Description:  Create or append a new file contraint (filter)
    ' Use:          AddFilterConstraint(<Description>,<Mask>)
    ' Examples:     AddFilterConstraint("Excel Files (*.xls)","*.xls")
    '               AddFilterConstraint("Pictures","*.bmp;*.gif;*.jpg")
    
    AddFilterConstraint = sOldFilter & sDescription & vbNullChar & sMask & vbNullChar
        
End Function


Private Function SnipAtNull(ByVal sString As String) As String

    '
    ' Description:  Cut off a string at it's NULL [CHR(0)]
    ' Use:          I would think it's quite obvious
    '

    SnipAtNull = IIf(InStr(sString, vbNullChar) > 0, Left(sString, InStr(sString, vbNullChar) - 1), sString)
   
       
End Function


